@@ -1,5 +1,9 @@
-#!/usr/bin/env python
-#Copyright (C) 2007-2008 Don Brown 2010 Spike Burch <spikeb@gmail.com>
+#!/usr/bin/env python2
+# Copyright (C)
+#    2007-2008 Don Brown,
+#    2010 Spike Burch <spikeb@gmail.com>,
+#    2015-2016 Vasya Novikov
+#
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation, either version 3 of the License, or
@@ -69,15 +73,14 @@ def load_sound(name):
     return sound
 
 
-# Loads a list of sounds
-def load_sounds(lst):
+def load_items(lst, blacklist, load_function):
+    """Runs load_function on elements of lst unless they are blacklisted."""
     result = []
-    global args
     for name in lst:
-        if True in [fnmatch.fnmatch(name, p) for p in args.sound_blacklist]:
-            print "Skipping blacklisted sound:", name
+        if True in [fnmatch.fnmatch(name, p) for p in blacklist]:
+            print "Skipping blacklisted item:", name
         else:
-            result.append(load_sound(name))
+            result.append(load_function(name))
     return result
 
 
@@ -88,8 +91,8 @@ def input(events, quit_pos):
         if event.type == QUIT: 
             sys.exit(0)
         
-        # handle keydown event     
-        elif event.type == KEYDOWN:
+        # handle keydown event
+        elif event.type == KEYDOWN or event.type == pygame.JOYBUTTONDOWN:
             # check for words like quit
             if event.type == KEYDOWN:
                 if is_alpha(event.key):
@@ -112,11 +115,14 @@ def input(events, quit_pos):
             
             # play random sound
             if not sound_muted:
-               sounds[random.randint(0, len(sounds) - 1)].play()
-            
-            # show images 
-            if is_alpha(event.key):
-                print_letter(event.key)
+                if event.type == KEYDOWN and args.deterministic_sounds:
+                    sounds[event.key % len(sounds)].play()
+                else:
+                    sounds[random.randint(0, len(sounds) -1)].play()
+
+            # show images
+            if event.type == pygame.KEYDOWN and (event.unicode.isalpha() or event.unicode.isdigit()):
+                print_letter(event.unicode)
             else:
                 print_image()
             pygame.display.flip()
@@ -153,13 +159,11 @@ def is_alpha(key):
     return key < 255 and (chr(key) in string.letters or chr(key) in string.digits)
 
 # Prints a letter at a random location
-def print_letter(key):
+def print_letter(char):
     global args
     font = pygame.font.Font(None, 256)
     if args.uppercase:
-        char = chr(key).upper()
-    else:
-        char = chr(key)
+        char = char.upper()
     text = font.render(char, 1, colors[random.randint(0, len(colors) - 1)])
     textpos = text.get_rect()
     center = (textpos.width / 2, textpos.height / 2)
@@ -174,6 +178,9 @@ def print_letter(key):
 parser = argparse.ArgumentParser(description='A keyboard mashing game for babies.')
 parser.add_argument('-u', '--uppercase', action='store_true', help='Whether to show UPPER-CASE letters.')
 parser.add_argument('--sound_blacklist', action='append', default=[], help='List of sound filename patterns to never play.')
+parser.add_argument('--image_blacklist', action='append', default=[], help='List of image filename patterns to never show.')
+parser.add_argument('-d', '--deterministic-sounds', action='store_true', help='Whether to produce same sounds on same key presses.')
+parser.add_argument('-m', '--mute', action='store_true', help='No sound will be played.')
 args = parser.parse_args()
 
 if not pygame.font: print 'Warning, fonts disabled'
@@ -182,7 +189,7 @@ if not pygame.mixer: print 'Warning, sound disabled'
 pygame.init()
 
 # figure out the install base to use with image and sound loading
-progInstallBase = os.path.dirname(os.path.normpath(sys.argv[0]));
+progInstallBase = os.path.dirname(os.path.realpath(sys.argv[0]));
 
 # swith to full screen at current screen resolution 
 window = pygame.display.set_mode((0, 0), pygame.FULLSCREEN)
@@ -198,30 +205,44 @@ screen = pygame.display.get_surface()
 background = pygame.Surface(screen.get_size())
 background = background.convert()
 background.fill((250, 250, 250))
+captionFont = pygame.font.SysFont(None, 20)
+captionLabel = captionFont.render("Commands: quit, mute, unmute", True, (210, 210, 210), (250, 250, 250))
+captionRect = captionLabel.get_rect()
+captionRect.x = 15
+captionRect.y = 10
+background.blit(captionLabel, captionRect)
 sequence = ""
 screen.blit(background, (0, 0))
 pygame.display.flip()
 
 mouse_down = False
-sound_muted = False
+sound_muted = args.mute
 
 def glob_data(pattern):
     return glob.glob(os.path.join(progInstallBase, 'data', pattern))
 
-sounds = load_sounds(glob_data('*.wav'))
+sounds = load_items(glob_data('*.wav'), args.sound_blacklist, load_sound)
 
 colors = ((  0,   0, 255), (255,   0,   0), (255, 255,   0), 
           (255,   0, 128), (  0,   0, 128), (  0, 255,   0), 
           (255, 128,   0), (255,   0, 255), (  0, 255, 255)
 )
 
-images = [load_image(glob_data('chimp.bmp')[0], -1)]
-images.extend([load_image(name) for name in glob_data('*.gif')])
+images = load_items(glob_data('*.gif'), args.image_blacklist, load_image)
 
 quit_pos = 0
 
 clock = pygame.time.Clock()
 
+pygame.joystick.init()
+
+# Initialize all joysticks
+joystick_count = pygame.joystick.get_count()
+for i in range(joystick_count):
+    joystick = pygame.joystick.Joystick(i)
+    joystick.init()
+    
+
 while True:
     clock.tick(60)
     quit_pos = input(pygame.event.get(), quit_pos)
